import 'package:masamune/masamune.dart';
import 'variable_config.dart';

@immutable
class SelectFormConfigBuilder extends FormConfigBuilder {
  const SelectFormConfigBuilder();
  @override
  bool check(FormConfig? form) {
    return form is SelectFormConfig;
  }

  @override
  Iterable<Widget> form(
    VariableConfig config,
    FormConfig? form,
    BuildContext context,
    WidgetRef ref, {
    DynamicMap? data,
    bool onlyRequired = false,
  }) {
    if (form is! SelectFormConfig) {
      return [];
    }
    return [
      if (config.label.isNotEmpty)
        DividHeadline(
          config.label.localize(),
          prefix: config.required ? context.widgetTheme.requiredIcon : null,
        )
      else
        const Divid(),
      FormItemDropdownField(
        dense: true,
        backgroundColor: form.backgroundColor,
        items: form.items,
        allowEmpty: !config.required,
        hintText: "Input %s".localize().format([config.label.localize()]),
        errorText: "No input %s".localize().format([config.label.localize()]),
        controller: ref.useTextEditingController(
            config.id, data.get(config.id, form.initialKey)),
        onSaved: (value) {
          if (value.isEmpty) {
            context[config.id] = form.initialKey;
          } else {
            context[config.id] = value;
          }
        },
      ),
    ];
  }

  @override
  Iterable<Widget> view(
    VariableConfig config,
    FormConfig? form,
    BuildContext context,
    WidgetRef ref, {
    DynamicMap? data,
    bool onlyRequired = false,
  }) {
    if (form is! SelectFormConfig) {
      return [];
    }
    return [
      if (config.label.isNotEmpty)
        DividHeadline(
          config.label.localize(),
        )
      else
        const Divid(),
      ListItem(
        title: Text(form.items[data.get(config.id, "")] ?? ""),
      ),
    ];
  }

  @override
  dynamic value(
    VariableConfig config,
    BuildContext context,
    WidgetRef ref,
  ) {
    return context.get(config.id, "");
  }
}
